@@ -57,6 +57,7 @@
 #define TextToggle		00000100000, 00000000000
 #define XfwfCommon		00000200000, 00000000000
 #define XfwfMenuBar		00000400000, 00000000000
+#define Tabs			00001000000, 00000000000
 
 #define ApplicationShell	Shell
 #define OverrideShell		Shell
@@ -160,6 +161,7 @@ struct resource_list {
 	{ "accelerators", "AcceleratorTable", SimpleMenu },
 	{ "accelerators", "AcceleratorTable", Slider2d },
 	{ "accelerators", "AcceleratorTable", StripChart },
+	{ "accelerators", "AcceleratorTable", Tabs },
 	{ "accelerators", "AcceleratorTable", TextBox },
 	{ "accelerators", "AcceleratorTable", TextButton },
 	{ "accelerators", "AcceleratorTable", TextToggle },
@@ -497,6 +499,7 @@ struct resource_list {
 	{ "borderWidth", "Dimension", SmeBSB },
 	{ "borderWidth", "Dimension", SmeLine },
 	{ "borderWidth", "Dimension", StripChart },
+	{ "borderWidth", "Dimension", Tabs },
 	{ "borderWidth", "Dimension", TextBox },
 	{ "borderWidth", "Dimension", TextButton },
 	{ "borderWidth", "Dimension", TextToggle },
@@ -530,6 +533,7 @@ struct resource_list {
 	{ "bottomShadowColor", "Pixel", TextButton },
 	{ "bottomShadowColor", "Pixel", TextToggle },
 	{ "bottomShadowColor", "Pixel", XfwfMenuBar },
+	{ "bottomShadowContast", "Int", Tabs },
 	{ "bottomShadowStipple", "Bitmap", Arrow },
 	{ "bottomShadowStipple", "Bitmap", Board },
 	{ "bottomShadowStipple", "Bitmap", Frame },
@@ -559,6 +563,7 @@ struct resource_list {
 	{ "callback", "Callback", Sme },
 	{ "callback", "Callback", SmeBSB },
 	{ "callback", "Callback", SmeLine },
+	{ "callback", "Callback", Tabs },
 	{ "callback", "Callback", Toggle },
 	{ "canvasHeight", "Dimension", Panner },
 	{ "canvasWidth", "Dimension", Panner },
@@ -640,6 +645,7 @@ struct resource_list {
 	{ "colormap", "Colormap", SimpleMenu },
 	{ "colormap", "Colormap", Slider2d },
 	{ "colormap", "Colormap", StripChart },
+	{ "colormap", "Colormap", Tabs },
 	{ "colormap", "Colormap", TextBox },
 	{ "colormap", "Colormap", TextButton },
 	{ "colormap", "Colormap", TextToggle },
@@ -865,6 +871,7 @@ struct resource_list {
 	{ "font", "FontStruct", Repeater },
 	{ "font", "FontStruct", Slider2d },
 	{ "font", "FontStruct", SmeBSB },
+	{ "font", "FontStruct", Tabs },
 	{ "font", "FontStruct", TextBox },
 	{ "font", "FontStruct", TextButton },
 	{ "font", "FontStruct", TextToggle },
@@ -1114,6 +1121,7 @@ struct resource_list {
 	{ "initialState", "InitialState", TopLevelShell },
 	{ "initialState", "InitialState", TransientShell },
 	{ "initialState", "InitialState", WMShell },
+	{ "insensitiveContrast", "Int", Tabs },
 	{ "innerOffset", "Dimension", Arrow },
 	{ "innerOffset", "Dimension", Board },
 	{ "innerOffset", "Dimension", Frame },
@@ -1182,6 +1190,7 @@ struct resource_list {
 	{ "internalHeight", "Dimension", List },
 	{ "internalHeight", "Dimension", MenuButton },
 	{ "internalHeight", "Dimension", Repeater },
+	{ "internalHeight", "Dimension", Tabs },
 	{ "internalHeight", "Dimension", Toggle },
 	{ "internalSpace", "Dimension", Panner },
 	{ "internalWidth", "Dimension", Command },
@@ -1189,6 +1198,7 @@ struct resource_list {
 	{ "internalWidth", "Dimension", List },
 	{ "internalWidth", "Dimension", MenuButton },
 	{ "internalWidth", "Dimension", Repeater },
+	{ "internalWidth", "Dimension", Tabs },
 	{ "internalWidth", "Dimension", Toggle },
 	{ "isIndex", "Boolean", HTML },
 	{ "italicFont", "FontStruct", HTML },
@@ -1463,6 +1473,7 @@ struct resource_list {
 	{ "pasteBuffer", "Boolean", List },
 	{ "pasteBuffer", "Boolean", MultiList },
 	{ "percentVerticalSpace", "Int", HTML },
+	{ "pickTop", "Boolean", Scrollbar },
 	{ "pieceSize", "Int", AsciiSrc },
 	{ "pieceSize", "Int", AsciiText },
 	{ "plainFont", "FontStruct", HTML },
@@ -1499,6 +1510,7 @@ struct resource_list {
 	{ "popdownCallback", "Callback", OverrideShell },
 	{ "popdownCallback", "Callback", Shell },
 	{ "popdownCallback", "Callback", SimpleMenu },
+	{ "popdownCallback", "Callback", Tabs },
 	{ "popdownCallback", "Callback", TopLevelShell },
 	{ "popdownCallback", "Callback", TransientShell },
 	{ "popdownCallback", "Callback", WMShell },
@@ -1506,6 +1518,7 @@ struct resource_list {
 	{ "popupCallback", "Callback", OverrideShell },
 	{ "popupCallback", "Callback", Shell },
 	{ "popupCallback", "Callback", SimpleMenu },
+	{ "popupCallback", "Callback", Tabs },
 	{ "popupCallback", "Callback", TopLevelShell },
 	{ "popupCallback", "Callback", TransientShell },
 	{ "popupCallback", "Callback", WMShell },
@@ -1573,6 +1586,7 @@ struct resource_list {
 	{ "reportCallback", "Callback", Viewport },
 	{ "resizable", "Boolean", Dialog },
 	{ "resizable", "Boolean", Form },
+	{ "resizable", "Boolean", Tabs },
 	{ "resizable", "Boolean", Viewport },
 	{ "resize", "Boolean", Command },
 	{ "resize", "Boolean", Label },
@@ -1679,6 +1693,7 @@ struct resource_list {
 	{ "scrollVertical", "ScrollMode", AsciiText },
 	{ "scrollbarForeground", "Pixel", Scrollbar2 },
 	{ "selectTypes", "Pointer", AsciiText },
+	{ "selectInsensitive", "Boolean", Tabs },
 	{ "selection", "Long", Group },
 	{ "selection", "Long", RadioGroup },
 	{ "selectionStyle", "SelectionType", Group },
@@ -1779,6 +1794,9 @@ struct resource_list {
 	{ "string", "String", AsciiSrc },
 	{ "string", "String", AsciiText },
 	{ "submitFormCallback", "Callback", HTML },
+	{ "tabForeground", "Pixel", Tabs },
+	{ "tabLabel", "String", Tabs },
+	{ "tabLeftBitmap", "Bitmap", Tabs },
 	{ "tablist", "String", MultiList },
 	{ "tablist", "String", Slider2d },
 	{ "tablist", "String", TextBox },
@@ -1826,6 +1844,7 @@ struct resource_list {
 	{ "topShadowColor", "Pixel", TextButton },
 	{ "topShadowColor", "Pixel", TextToggle },
 	{ "topShadowColor", "Pixel", XfwfMenuBar },
+	{ "topShadowContast", "Int", Tabs },
 	{ "topShadowStipple", "Bitmap", Arrow },
 	{ "topShadowStipple", "Bitmap", Board },
 	{ "topShadowStipple", "Bitmap", Frame },
@@ -1839,6 +1858,7 @@ struct resource_list {
 	{ "topShadowStipple", "Bitmap", TextButton },
 	{ "topShadowStipple", "Bitmap", TextToggle },
 	{ "topShadowStipple", "Bitmap", XfwfMenuBar },
+	{ "topWidget", "Widget", Tabs },
 	{ "transient", "Boolean", ApplicationShell },
 	{ "transient", "Boolean", TopLevelShell },
 	{ "transient", "Boolean", TransientShell },
